﻿local _G = _G
local select = _G.select
local pairs = _G.pairs
local ipairs = _G.ipairs
local string = _G.string
local type = _G.type
local error = _G.error
local table = _G.table
local C_PetJournal = _G.C_PetJournal
local C_ToyBox = _G.C_ToyBox

ArkInventory.Toybox = { }

local filter = {
	ignore = false,
	searchBox = nil,
	collected = true,
	notcollected = true,
	source = { },
}

local collection = {
	loaded = false,
	total = 0, -- number of total toys
	owned = 0, -- number of owned toys
	cache = { },	-- [ID] = { } - owned toys only
}


function ArkInventory.Toybox.FilterGetSourceTypes( )
	return C_PetJournal.GetNumPetSources( )
end

function ArkInventory.Toybox.FilterGetSearch( )
	return ToyBox.searchBox:GetText( )
end

function ArkInventory.Toybox.FilterSetSearch( s )
	ToyBox.searchBox:SetText( s )
	C_ToyBox.SetFilterString( s )
end

function ArkInventory.Toybox.FilterSetCollected( value )
	C_ToyBox.SetCollectedShown( value )
	-- legion ok
end

function ArkInventory.Toybox.FilterGetCollected( )
	return C_ToyBox.GetCollectedShown( )
	-- legion ok
end

function ArkInventory.Toybox.FilterSetUncollected( value )
	C_ToyBox.SetUncollectedShown( value )
	-- legion ok
end

function ArkInventory.Toybox.FilterGetUncollected( )
	return C_ToyBox.GetUncollectedShown( )
	-- legion ok
end

function ArkInventory.Toybox.FilterSetSource( t )
	if type( t ) == "table" then
		for i = 1, ArkInventory.Toybox.FilterGetSourceTypes( ) do
			C_ToyBox.SetSourceTypeFilter( i, t[i] )
		end
	elseif type( t ) == "boolean" then
		for i = 1, ArkInventory.Toybox.FilterGetSourceTypes( ) do
			C_ToyBox.SetSourceTypeFilter( i, t )
		end
	else
		assert( false, "parameter is not a table or boolean" )
	end
	-- legion ok
end

function ArkInventory.Toybox.FilterGetSource( t )
	assert( type( t ) == "table", "parameter is not a table" )
	for i = 1, ArkInventory.Toybox.FilterGetSourceTypes( ) do
		t[i] = C_ToyBox.IsSourceTypeFilterChecked( i )
	end
	-- legion ok
end

function ArkInventory.Toybox.OnHide( )
	filter.ignore = false
	ArkInventory:SendMessage( "EVENT_ARKINV_TOYBOX_RELOAD_BUCKET", "RESCAN" )
end

function ArkInventory.Toybox.FilterActionClear( )
	
	filter.ignore = true
	
	ArkInventory.Toybox.FilterSetSearch( "" )
	ArkInventory.Toybox.FilterSetCollected( true )
	ArkInventory.Toybox.FilterSetUncollected( true )
	ArkInventory.Toybox.FilterSetSource( true )
	
end

function ArkInventory.Toybox.FilterActionBackup( )
	
	--ArkInventory.Output( "Toybox.FilterActionBackup" )
	
	if filter.ignore then
		--ArkInventory.Output( "FilterActionBackup - ignore" )
		return
	end
	
	filter.search = ArkInventory.Toybox.FilterGetSearch( )
	filter.collected = ArkInventory.Toybox.FilterGetCollected( )
	filter.uncollected = ArkInventory.Toybox.FilterGetUncollected( )
	ArkInventory.Toybox.FilterGetSource( filter.source )
	
end

function ArkInventory.Toybox.FilterActionRestore( )
	
	--ArkInventory.Output( "Toybox.FilterActionRestore" )
	
	filter.ignore = true
	
	ArkInventory.Toybox.FilterSetSearch( filter.search )
	ArkInventory.Toybox.FilterSetCollected( filter.collected )
	ArkInventory.Toybox.FilterSetUncollected( filter.uncollected )
	ArkInventory.Toybox.FilterSetSource( filter.source )
	
end



function ArkInventory.Toybox.JournalIsReady( )
	return collection.loaded
end

function ArkInventory.Toybox.GetCount( )
	return collection.owned, collection.total
end

function ArkInventory.Toybox.GetToy( value )
	
	if type( value ) == "number" then
		return collection.cache[value]
	end
	
end

function ArkInventory.Toybox.Iterate( )
	local t = collection.cache
	return ArkInventory.spairs( t, function( a, b ) return ( t[a].fullname or "" ) < ( t[b].fullname or "" ) end )
end

function ArkInventory.Toybox.Summon( index )
	local td = ArkInventory.Toybox.GetToy( index )
	UseToy( td.item )
end

function ArkInventory.Toybox.GetFavorite( index )
	-- index = item id
	local td = ArkInventory.Toybox.GetToy( index )
	return C_ToyBox.GetIsFavorite( td.item )
end

function ArkInventory.Toybox.SetFavorite( index, value )
	-- value = true | false
	local td = ArkInventory.Toybox.GetToy( index )
	C_ToyBox.SetIsFavorite( td.item, value )
end

function ArkInventory.Toybox.Scan( )
	
	--ArkInventory.Output( "ArkInventory.Toybox.Scan( )" )
	
	if ArkInventory.Global.Mode.Combat then
		-- set to scan when leaving combat
		ArkInventory.Global.LeaveCombatRun.Toybox = true
		return
	end
	
	local total = C_ToyBox.GetNumTotalDisplayedToys( )
	local owned = C_ToyBox.GetNumLearnedDisplayedToys( )
	--ArkInventory.Output( "toys: ", owned, " of ", total )
	
	if owned == 0 then return end
	
	ArkInventory.Toybox.FilterActionBackup( )
	ArkInventory.Toybox.FilterActionClear( )
	
	local update = false
	
	if collection.total ~= total or not collection.loaded then
		collection.total = total
		update = true
	end
		
	if collection.owned ~= owned or not collection.loaded then
		
		collection.loaded = true
		
		collection.owned = owned
		update = true
		
		wipe( collection.cache )
		
	end
	
	local td = collection.cache
	
   for i = 1, total do
		
		local item = C_ToyBox.GetToyFromIndex( i )
		local owned = PlayerHasToy( item )
		
		if owned then
			
			local item, name, icon = C_ToyBox.GetToyInfo( item )
			--ArkInventory.Output( item, " / ", name )
			local fav = C_ToyBox.GetIsFavorite( item )
			
			if ( not update ) and ( ( not td[i] ) or ( td[i].name ~= name ) or ( td[i].fav ~= fav ) ) then
				update = true
			end
			
			td[i] = {
				link = C_ToyBox.GetToyLink( item ),
				item = item,
				index = i,
				name = name,
				texture = icon,
				fav = fav,
			}
			
		end
		
	end
	
	ArkInventory.Toybox.FilterActionRestore( )
	
	if update then
		ArkInventory.ScanToybox( )
	end
	
	return true
	
end



function ArkInventory:EVENT_WOW_COLLECTION_TOY_RELOAD( event, ... )
	
	--ArkInventory.Output( "EVENT_WOW_COLLECTION_TOY_RELOAD( ", event, " )" )
	
	ArkInventory:SendMessage( "EVENT_ARKINV_TOYBOX_RELOAD_BUCKET", event )
	
end

function ArkInventory:EVENT_ARKINV_TOYBOX_RELOAD_BUCKET( events )
	
	--ArkInventory.Output( "EVENT_ARKINV_TOYBOX_RELOAD_BUCKET( ", events, " )" )
	
	loc_id = ArkInventory.Const.Location.Toybox
	
	if not ArkInventory:IsEnabled( ) then
		return
	end
	
	if not ArkInventory.LocationIsMonitored( loc_id ) then
		--ArkInventory.Output( "IGNORED (NOT MONITORED)" )
		return
	end
	
	if ToyBox:IsVisible( ) then
		--ArkInventory.Output( "IGNORED (COLLECTION OPEN)" )
		return
	end
	
	if filter.ignore then
		--ArkInventory.Output( "IGNORED (FILTER CHANGED BY ME)" )
		filter.ignore = false
		return
	end
	
	ArkInventory.Toybox.Scan( )
	
end
